/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.peoplelist.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.base.peoplelist.entity.PeopleList;
import vip.xiaonuo.modular.base.peoplelist.param.PeopleListAddParam;
import vip.xiaonuo.modular.base.peoplelist.param.PeopleListEditParam;
import vip.xiaonuo.modular.base.peoplelist.param.PeopleListIdParam;
import vip.xiaonuo.modular.base.peoplelist.param.PeopleListPageParam;
import vip.xiaonuo.modular.base.peoplelist.service.PeopleListService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.Date;

/**
 * people_list控制器
 *
 * @author wang
 * @date 2023/12/28 16:19
 */
@Api(tags = "人员管理控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class PeopleListController {

    @Resource
    private PeopleListService peopleListService;

    /**
     * 获取people_list分页
     *
     * @author wang
     * @date 2023/12/28 16:19
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取people_list分页")
    @GetMapping("/base/peoplelist/page")
    public CommonResult<Page<PeopleList>> page(PeopleListPageParam peopleListPageParam) {
        return CommonResult.data(peopleListService.page(peopleListPageParam));
    }

    /**
     * 添加people_list
     *
     * @author wang
     * @date 2023/12/28 16:19
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加people_list")
    @CommonLog("添加people_list")
    @PostMapping("/base/peoplelist/add")
    public CommonResult<String> add(@RequestParam @ApiParam(value = "姓名") String userName,
                                    @RequestParam @ApiParam(value = "工号") String nickName,
                                    @RequestParam @ApiParam(value = "性别 0为女 1为男") Integer sex,
                                    @RequestParam @ApiParam(value = "身份证") String idCard,
                                    @RequestParam @ApiParam(value = "员工电话") String phone,
                                    @RequestParam @ApiParam(value = "紧急联系人电话") String publicPhone,
                                    @RequestParam @ApiParam(value = "省市区") String province,
                                    @RequestParam @ApiParam(value = "详细地址") String address,
                                    @RequestParam @ApiParam(value = "入职时间") String entryDate,
                                    @RequestParam @ApiParam(value = "0 在职 1离职") Integer workStatus,
                                    @RequestParam @ApiParam(value = "工作岗位") String job,
                                    @RequestParam @ApiParam(value = "部门") String department,
                                    @RequestParam @ApiParam(value = "工作地点") String workLocation,
                                    @RequestParam @ApiParam(value = "员工照片") MultipartFile imgUrl,
                                    @RequestParam @ApiParam(value = "组织id") String orgId) {
        peopleListService.add(userName, nickName, sex, idCard, phone, publicPhone, province, address, entryDate,
                workStatus, job, department, workLocation, imgUrl,orgId);
        return CommonResult.ok();
    }

    /**
     * 编辑people_list
     *
     * @author wang
     * @date 2023/12/28 16:19
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑people_list")
    @CommonLog("编辑people_list")
    @PostMapping("/base/peoplelist/edit")
    public CommonResult<String> edit(@RequestParam @ApiParam(value = "id") String id,
                                     @RequestParam @ApiParam(value = "姓名") String userName,
                                     @RequestParam @ApiParam(value = "工号") String nickName,
                                     @RequestParam @ApiParam(value = "性别 0为女 1为男") Integer sex,
                                     @RequestParam @ApiParam(value = "身份证") String idCard,
                                     @RequestParam @ApiParam(value = "员工电话") String phone,
                                     @RequestParam @ApiParam(value = "紧急联系人电话") String publicPhone,
                                     @RequestParam @ApiParam(value = "省市区") String province,
                                     @RequestParam @ApiParam(value = "详细地址") String address,
                                     @RequestParam @ApiParam(value = "入职时间") String entryDate,
                                     @RequestParam @ApiParam(value = "0 在职 1离职") Integer workStatus,
                                     @RequestParam @ApiParam(value = "工作岗位") String job,
                                     @RequestParam @ApiParam(value = "部门") String department,
                                     @RequestParam @ApiParam(value = "工作地点") String workLocation,
                                     @RequestParam @ApiParam(value = "员工照片") MultipartFile imgUrl) {
        peopleListService.edit(id,userName, nickName, sex, idCard, phone, publicPhone, province, address, entryDate,
                workStatus, job, department, workLocation,  imgUrl);
        return CommonResult.ok();
    }

    /**
     * 删除people_list
     *
     * @author wang
     * @date 2023/12/28 16:19
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除people_list")
    @CommonLog("删除people_list")
    @PostMapping("/base/peoplelist/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<PeopleListIdParam> peopleListIdParamList) {
        peopleListService.delete(peopleListIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取people_list详情
     *
     * @author wang
     * @date 2023/12/28 16:19
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取people_list详情")
    @GetMapping("/base/peoplelist/detail")
    public CommonResult<PeopleList> detail(@Valid PeopleListIdParam peopleListIdParam) {
        return CommonResult.data(peopleListService.detail(peopleListIdParam));
    }
}
