/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigtransfer.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.bi.pigtransfer.entity.PigTransfer;
import vip.xiaonuo.modular.bi.pigtransfer.mapper.PigTransferMapper;
import vip.xiaonuo.modular.bi.pigtransfer.param.PigTransferAddParam;
import vip.xiaonuo.modular.bi.pigtransfer.param.PigTransferEditParam;
import vip.xiaonuo.modular.bi.pigtransfer.param.PigTransferIdParam;
import vip.xiaonuo.modular.bi.pigtransfer.param.PigTransferPageParam;
import vip.xiaonuo.modular.bi.pigtransfer.service.PigTransferService;

import java.util.Date;
import java.util.List;

/**
 * 转舍信息Service接口实现类
 *
 * @author wang
 * @date  2023/12/07 18:02
 **/
@Service
public class PigTransferServiceImpl extends ServiceImpl<PigTransferMapper, PigTransfer> implements PigTransferService {

    @Override
    public Page<PigTransfer> page(PigTransferPageParam pigTransferPageParam) {
        QueryWrapper<PigTransfer> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(pigTransferPageParam.getEartag())) {
            queryWrapper.lambda().like(PigTransfer::getEartag, pigTransferPageParam.getEartag());
        }
        if(ObjectUtil.isNotEmpty(pigTransferPageParam.getStartIntoDate()) && ObjectUtil.isNotEmpty(pigTransferPageParam.getEndIntoDate())) {
            queryWrapper.lambda().between(PigTransfer::getIntoDate, pigTransferPageParam.getStartIntoDate(), pigTransferPageParam.getEndIntoDate());
        }
        if(ObjectUtil.isAllNotEmpty(pigTransferPageParam.getSortField(), pigTransferPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigTransferPageParam.getSortOrder());
            queryWrapper.orderBy(true, pigTransferPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigTransferPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigTransfer::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PigTransferAddParam pigTransferAddParam) {
        PigTransfer pigTransfer = BeanUtil.toBean(pigTransferAddParam, PigTransfer.class);
        pigTransfer.setCreateDate(new Date());
        this.save(pigTransfer);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(PigTransferEditParam pigTransferEditParam) {
        PigTransfer pigTransfer = this.queryEntity(pigTransferEditParam.getId());
        BeanUtil.copyProperties(pigTransferEditParam, pigTransfer);
        this.updateById(pigTransfer);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigTransferIdParam> pigTransferIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(pigTransferIdParamList, PigTransferIdParam::getId));
    }

    @Override
    public PigTransfer detail(PigTransferIdParam pigTransferIdParam) {
        return this.queryEntity(pigTransferIdParam.getId());
    }

    @Override
    public PigTransfer queryEntity(Integer id) {
        PigTransfer pigTransfer = this.getById(id);
        if(ObjectUtil.isEmpty(pigTransfer)) {
            throw new CommonException("转舍信息不存在，id值为：{}", id);
        }
        return pigTransfer;
    }
}
