/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.energy.energyDevice.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.base.baseConfig.entity.BaseConfig;
import vip.xiaonuo.modular.base.pigpen.param.BasePigpenOrgIdParam;
import vip.xiaonuo.modular.base.pigpen.service.BasePigpenService;
import vip.xiaonuo.modular.energy.energyDevice.entity.EnergyDevice;
import vip.xiaonuo.modular.energy.energyDevice.entity.vo.EnergyDeviceCountVo;
import vip.xiaonuo.modular.energy.energyDevice.param.*;
import vip.xiaonuo.modular.energy.energyDevice.service.EnergyDeviceService;
import vip.xiaonuo.modular.env.envdevice.param.EnvDeviceOrgIdParam;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;
import java.util.Map;

/**
 * 能耗设备控制器
 *
 * @author newspaper
 * @date  2023/12/28 08:46
 */
@Api(tags = "能耗设备控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class EnergyDeviceController {

    @Resource
    private EnergyDeviceService energyDeviceService;
    @Resource
    private BasePigpenService basePigpenService;

    /**
     * 获取能耗设备分页
     *
     * @author newspaper
     * @date  2023/12/28 08:46
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取能耗设备分页")
    @GetMapping("/energy/energyDevice/page")
    public CommonResult<Page<EnergyDevice>> page(EnergyDevicePageParam energyDevicePageParam) {
        return CommonResult.data(energyDeviceService.page(energyDevicePageParam));
    }

    /**
     * 添加能耗设备
     *
     * @author newspaper
     * @date  2023/12/28 08:46
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加能耗设备")
    @CommonLog("添加能耗设备")
    @PostMapping("/energy/energyDevice/add")
    public CommonResult<String> add(@RequestBody @Valid EnergyDeviceAddParam energyDeviceAddParam) {
        energyDeviceService.add(energyDeviceAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑能耗设备
     *
     * @author newspaper
     * @date  2023/12/28 08:46
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑能耗设备")
    @CommonLog("编辑能耗设备")
    @PostMapping("/energy/energyDevice/edit")
    public CommonResult<String> edit(@RequestBody @Valid EnergyDeviceEditParam energyDeviceEditParam) {
        energyDeviceService.edit(energyDeviceEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除能耗设备
     *
     * @author newspaper
     * @date  2023/12/28 08:46
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除能耗设备")
    @CommonLog("删除能耗设备")
    @PostMapping("/energy/energyDevice/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<EnergyDeviceIdParam> energyDeviceIdParamList) {
        energyDeviceService.delete(energyDeviceIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取能耗设备详情
     *
     * @author newspaper
     * @date  2023/12/28 08:46
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取能耗设备详情")
    @GetMapping("/energy/energyDevice/detail")
    public CommonResult<EnergyDevice> detail(@Valid EnergyDeviceIdParam energyDeviceIdParam) {
        return CommonResult.data(energyDeviceService.detail(energyDeviceIdParam));
    }


    /**
     * 获取环控设备汇总
     *
     * @author newspaper
     * @date  2023/12/28 08:46
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("获取能耗设备汇总")
    @GetMapping("/energy/energyDevice/deviceCount")
    public CommonResult<EnergyDeviceCountVo> deviceCount(EnergyDeviceOrgIdParam energyDeviceOrgIdParam) {
        return CommonResult.data(energyDeviceService.deviceCount(energyDeviceOrgIdParam));
    }


    /**
     * 获取栋舍选择器
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("获取栋舍选择器")
    @GetMapping("/energy/energyDevice/pipgpenSelector")
    public CommonResult<List<Tree<String>>> pipgpenSelector(BasePigpenOrgIdParam basePigpenOrgIdParam) {
        return CommonResult.data(basePigpenService.tree(basePigpenOrgIdParam));
    }

    /**
     * 获取设备类型选择器
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 8)
    @ApiOperation("获取设备类型选择器")
    @GetMapping("/energy/energyDevice/deviceTypeSelector")
    public CommonResult<List<BaseConfig>> deviceTypeSelector(EnergyDeviceOrgIdParam energyDeviceOrgIdParam) {
        return CommonResult.data(energyDeviceService.deviceTypeSelector(energyDeviceOrgIdParam));
    }


}
