/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.env.envdata.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.env.envdata.entity.vo.EnvDataRecentVo;
import vip.xiaonuo.modular.env.envdata.entity.vo.EnvDataTrendVo;
import vip.xiaonuo.modular.env.envdata.param.EnvDataListParm;
import vip.xiaonuo.modular.env.envdata.param.EnvDataPigpenIdParm;
import vip.xiaonuo.modular.env.envdata.service.EnvDataService;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * 环控数据控制器
 *
 * @author newspaper
 * @date  2023/12/20 14:38
 */
@Api(tags = "环控数据控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class EnvDataController {
    @Autowired
    private EnvDataService envDataService;

    /**
     * 根据楼层ID获取最新环控数据
     *
     * @author newspaper
     * @date  2023/12/20 14:38
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("根据楼层ID获取最新环控数据")
    @GetMapping("/env/envData/getLastDataByPigpenId")
    public CommonResult<List<EnvDataRecentVo>> getLastDataByPigpenId(EnvDataPigpenIdParm envDataPigpenIdParm){
        return CommonResult.data(envDataService.getLastDataByPigpenId(envDataPigpenIdParm));
    }

    /**
     * 根据单元ID获取环控数据
     *
     * @author newspaper
     * @date  2023/12/20 14:38
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("根据单元ID获取环控数据")
    @PostMapping("/env/envData/listDataByPigpenId")
    public CommonResult<EnvDataTrendVo> listDataByPigpenId(@RequestBody @Valid EnvDataListParm envDataListParm){
        return CommonResult.data(envDataService.listDataByPigpenId(envDataListParm));
    }


}
