/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.env.envdevice.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.base.baseConfig.entity.BaseConfig;
import vip.xiaonuo.modular.base.pigpen.param.BasePigpenOrgIdParam;
import vip.xiaonuo.modular.base.pigpen.service.BasePigpenService;
import vip.xiaonuo.modular.env.envdevice.entity.EnvDevice;
import vip.xiaonuo.modular.env.envdevice.entity.vo.EnvDeviceCountVo;
import vip.xiaonuo.modular.env.envdevice.param.*;
import vip.xiaonuo.modular.env.envdevice.service.EnvDeviceService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;
import java.util.Map;

/**
 * 环控设备控制器
 *
 * @author newspaper
 * @date  2023/12/18 09:58
 */
@Api(tags = "环控设备控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class EnvDeviceController {

    @Resource
    private EnvDeviceService envDeviceService;
    @Resource
    private BasePigpenService basePigpenService;

    /**
     * 获取环控设备分页
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取环控设备分页")
    @GetMapping("/env/envDevice/page")
    public CommonResult<Page<EnvDevice>> page(EnvDevicePageParam envDevicePageParam) {
        return CommonResult.data(envDeviceService.page(envDevicePageParam));
    }

    /**
     * 添加环控设备
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加环控设备")
    @CommonLog("添加环控设备")
    @PostMapping("/env/envDevice/add")
    public CommonResult<String> add(@RequestBody @Valid EnvDeviceAddParam envDeviceAddParam) {
        envDeviceService.add(envDeviceAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑环控设备
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑环控设备")
    @CommonLog("编辑环控设备")
    @PostMapping("/env/envDevice/edit")
    public CommonResult<String> edit(@RequestBody @Valid EnvDeviceEditParam envDeviceEditParam) {
        envDeviceService.edit(envDeviceEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除环控设备
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除环控设备")
    @CommonLog("删除环控设备")
    @PostMapping("/env/envDevice/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<EnvDeviceIdParam> envDeviceIdParamList) {
        envDeviceService.delete(envDeviceIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取环控设备详情
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取环控设备详情")
    @GetMapping("/env/envDevice/detail")
    public CommonResult<EnvDevice> detail(@Valid EnvDeviceIdParam envDeviceIdParam) {
        return CommonResult.data(envDeviceService.detail(envDeviceIdParam));
    }

    /**
     * 获取环控设备汇总
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("获取环控设备汇总")
    @GetMapping("/env/envDevice/deviceCount")
    public CommonResult<EnvDeviceCountVo> deviceCount(EnvDeviceOrgIdParam envDeviceOrgIdParam) {
        return CommonResult.data(envDeviceService.deviceCount(envDeviceOrgIdParam));
    }

    /**
     * 获取栋舍选择器
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("获取栋舍选择器")
    @GetMapping("/env/envDevice/pipgpenSelector")
    public CommonResult<List<Tree<String>>> pipgpenSelector(BasePigpenOrgIdParam basePigpenOrgIdParam) {
        return CommonResult.data(basePigpenService.tree(basePigpenOrgIdParam));
    }

    /**
     * 获取设备类型选择器
     *
     * @author newspaper
     * @date  2023/12/18 09:58
     */
    @ApiOperationSupport(order = 8)
    @ApiOperation("获取设备类型选择器")
    @GetMapping("/env/envDevice/deviceTypeSelector")
    public CommonResult<List<BaseConfig>> deviceTypeSelector(EnvDeviceOrgIdParam envDeviceOrgIdParam) {
        return CommonResult.data(envDeviceService.deviceTypeSelector(envDeviceOrgIdParam));
    }
}
