/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.warning.warningInfo.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.warning.warningInfo.entity.WarningInfo;
import vip.xiaonuo.modular.warning.warningInfo.entity.vo.WarningInfoListVo;
import vip.xiaonuo.modular.warning.warningInfo.entity.vo.WarningInfoProportionVo;
import vip.xiaonuo.modular.warning.warningInfo.param.*;
import vip.xiaonuo.modular.warning.warningInfo.service.WarningInfoService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 预警控制器
 *
 * @author newspaper
 * @date  2023/12/26 13:40
 */
@Api(tags = "预警控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class WarningInfoController {

    @Resource
    private WarningInfoService warningInfoService;

    /**
     * 获取预警分页
     *
     * @author newspaper
     * @date  2023/12/26 13:40
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取预警分页")
    @GetMapping("/warning/warningInfo/page")
    public CommonResult<Page<WarningInfo>> page(WarningInfoPageParam warningInfoPageParam) {
        return CommonResult.data(warningInfoService.page(warningInfoPageParam));
    }

    /**
     * 获取预警占比
     *
     * @author newspaper
     * @date  2023/12/26 13:40
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("获取预警占比")
    @GetMapping("/warning/warningInfo/getProportion")
    public CommonResult<WarningInfoProportionVo> getProportion(WarningInfoOrgIdParam warningInfoOrgIdParam) {
        return CommonResult.data(warningInfoService.getProportion(warningInfoOrgIdParam));
    }

    /**
     * 获取近七天预警
     *
     * @author newspaper
     * @date  2023/12/26 13:40
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("获取近七天预警")
    @GetMapping("/warning/warningInfo/listSevenDayWarning")
    public CommonResult<List<WarningInfoListVo>> listSevenDayWarning(WarningInfoOrgIdParam warningInfoOrgIdParam) {
        return CommonResult.data(warningInfoService.listSevenDayWarning(warningInfoOrgIdParam));
    }



}
